//
//  FlipsideViewController.m
//  MapCast
//
//  Created by Yos Hashimoto.
//  Copyright Newton Japan Inc. 2009. All rights reserved.
//

#import "FlipsideViewController.h"


@implementation FlipsideViewController

@synthesize delegate, appData, textView, imageView;

#pragma mark -
#pragma mark Object lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor viewFlipsideBackgroundColor];      
}

- (void)viewDidAppear: (Boolean)animated
{
	[super viewDidAppear:animated];
	
	[imageView setImage:appData.cameraImage];
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

#pragma mark -
#pragma mark Flip back

- (IBAction)done {
	[self.delegate flipsideViewControllerDidFinish:self];	
}

#pragma mark -
#pragma mark Shrink image

-(UIImage*)shrinkImage:(UIImage*)image {
	UIImage*    shrinkedImage;

	CGSize  size = { 400, 300 };
	UIGraphicsBeginImageContext(size);
	CGRect  rect;
	rect.origin = CGPointZero;
	rect.size = size;
	[image drawInRect:rect];
	shrinkedImage = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	
	return shrinkedImage;
}

#pragma mark -
#pragma mark Connect to server

- (IBAction)send {
	[self sendImage:appData.cameraImage andText:[textView text]];
}


#define ASCII_DATA(str) [(NSString*)(str) dataUsingEncoding:NSASCIIStringEncoding]
- (void) sendImage:(UIImage*)imageToSend andText:(NSString*)comment {
	
	[UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
	
	NSString* freeComment = comment;
	if(!freeComment) freeComment = @"";
	
	// iPhoneデバイスIDの準備
	UIDevice *dev = [UIDevice currentDevice];
	NSString *uniqueId = dev.uniqueIdentifier;
	
	// 送信する画像データの準備
	NSData * postData = UIImageJPEGRepresentation([self shrinkImage:imageToSend], 100); 
	
	// URLリクエストを準備する
	NSURL *cgiUrl = [NSURL URLWithString:@"http://newtonjapan.com/book/demo/MAPCAST/info_upload.php"];
	//NSURL *cgiUrl = [NSURL URLWithString:@"http://newtonjapan.com/MAPCAST/info_upload.php"];
	NSMutableURLRequest *postRequest = [NSMutableURLRequest requestWithURL:cgiUrl]; 
	[postRequest setHTTPMethod:@"POST"]; 
	
 	//setting up the body:
    NSMutableData* result = [[NSMutableData alloc] init];	// Bodyを作成するための NSData
	NSString* boundary = @"___boundary___";
	
	// FreeCommentをURLエンコードする
	CFStringRef tmp = CFURLCreateStringByReplacingPercentEscapesUsingEncoding(kCFAllocatorDefault, (CFStringRef)freeComment, CFSTR(""), kCFStringEncodingUTF8);
	CFStringRef escapedStr1 = CFURLCreateStringByAddingPercentEscapes(kCFAllocatorDefault, tmp, nil, nil, kCFStringEncodingUTF8);	
	NSString* escapedStr10 = (NSString*)escapedStr1;	
	
	// 写真データ
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"; filename=\"%@\"\n", @"uploaded", @"upload.JPG"]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"Content-Type: image/jpeg\n\n"]))];
	[result appendData:postData];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	// コメント
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\n\n", @"freecomment"]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"%@", escapedStr10]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	
	// 緯度
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\n\n", @"lat"]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"%.6f", appData.coordinate.latitude]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	// 経度
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\n\n", @"lon"]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"%.6f", appData.coordinate.longitude]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	// 方角
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\n\n", @"heading"]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"%.6f", appData.heading]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	
	// 日付時刻（YYYY-MM-DD-HH-MM-SS）
	NSString *formattedDateString = [[NSDate date] descriptionWithCalendarFormat:@"%Y-%m-%d-%H-%M-%S" timeZone:nil locale:nil];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\n\n", @"datetime"]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"%@",formattedDateString]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	// デバイスID
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@\n", boundary]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\n\n", @"device_id"]))];
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"%@",uniqueId]))];
	[result appendData:ASCII_DATA(([NSString stringWithString:@"\n"]))];
	
	// マルチパート終端
	[result appendData:ASCII_DATA(([NSString stringWithFormat:@"--%@--\n\n", boundary]))];
	
	// POST実行
	[postRequest addValue: @"multipart/form-data; boundary=___boundary___" forHTTPHeaderField: @"Content-Type"];
    [postRequest setHTTPBody:result];
	
	NSURLConnection *conn = [[NSURLConnection alloc] initWithRequest:postRequest delegate:self];
	if (!conn)
	{
		NSLog(@"データの送信失敗");
	}
}


- (void) connection : (NSURLConnection *) connection
 didReceiveResponse : (NSURLResponse *) response {
	
	NSDictionary *dicHead = [ (NSHTTPURLResponse *) response allHeaderFields ];
	//----- HTTPヘッダー全体の表示 -----//
	NSEnumerator *enumHead = [ dicHead keyEnumerator ];
	NSMutableString *sHead = [ NSMutableString string ];
	id key;
	
	while ( ( key = [ enumHead nextObject ] ) ) {
		NSString *sValue = [ dicHead objectForKey : key ]; 
		[ sHead appendFormat : @"%@ = %@", key, sValue ];
		[ sHead appendFormat : @"%C", 0x000D ]; // 改行
	}
	NSLog(sHead);
	
	UIAlertView *openURLAlert = [[UIAlertView alloc] initWithTitle:@"情報アップロード" message:@"情報をサーバーに送信しました" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [openURLAlert show];
    [openURLAlert release];

	[UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}




@end
