//
//  MainViewController.m
//  MapCast
//
//  Created by Yos Hashimoto.
//  Copyright Newton Japan Inc. 2009. All rights reserved.
//

#import "MainViewController.h"
#import "MainView.h"


@implementation MainViewController

@synthesize mapview, locationLabel, appData;

#pragma mark -
#pragma mark Object lifecycle

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}


- (void)viewDidLoad {
	[super viewDidLoad];
	
	// Mapの縮尺を設定
	MKCoordinateRegion zoom = mapview.region;
	zoom.span.latitudeDelta = 0.005;
	zoom.span.longitudeDelta = 0.005;
	[mapview setRegion:zoom animated:YES];
	
	// CLLocationManagerのインスタンスを作成
	CLLocationManager *locationManager = [[CLLocationManager alloc] init];
	if ([locationManager locationServicesEnabled]) {
		locationManager.delegate = self;
		locationManager.desiredAccuracy = kCLLocationAccuracyKilometer;
		locationManager.distanceFilter = 1000.0;
		[locationManager startUpdatingLocation];
		[locationManager startUpdatingHeading];
	} else {
		NSLog(@"ロケーションサービスが利用できない");
	}
}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}


- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


#pragma mark -
#pragma mark FlipsideView

- (void)flipsideViewControllerDidFinish:(FlipsideViewController *)controller {
    
	[self dismissModalViewControllerAnimated:YES];
}


- (IBAction)showInfo {    
	
	FlipsideViewController *controller = [[FlipsideViewController alloc] initWithNibName:@"FlipsideView" bundle:nil];
	controller.delegate = self;
	controller.appData = appData;
	
	controller.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
	[self presentModalViewController:controller animated:YES];
	
	[controller release];
}


#pragma mark -
#pragma mark imagePickerController & delegate

- (IBAction) cameraButtonPushed {
	UIImagePickerController *picker = [[UIImagePickerController alloc] init];  
	picker.delegate = self;  
	picker.sourceType = UIImagePickerControllerSourceTypeCamera;
	//	picker.allowsImageEditing = YES;
	
	[self presentModalViewController:picker animated:YES];
	[picker release];
}


/* 以下のメソッドは今後のSDKで使われなくなる可能性あり
 - (void) imagePickerController:(UIImagePickerController*)picker  
 didFinishPickingImage:(UIImage *)image editingInfo:(NSDictionary*)editingInfo {  
 // image を処理  
 [picker dismissModalViewControllerAnimated:YES];  
 }
 */

// SDK3.0では、こちらを使うこと
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info {
	
	[[picker parentViewController] dismissModalViewControllerAnimated:YES];
	UIImage * image = [info objectForKey:@"UIImagePickerControllerOriginalImage"];
	//	UIImage * image = [info objectForKey:@"UIImagePickerControllerEditedImage"];
	if(image!=nil) {
		appData.cameraImage = image;
		//[appData setCameraImage:image];
		[self showInfo];
	}
}

- (void) imagePickerControllerDidCancel:(UIImagePickerController*)picker {  
    //キャンセル時の処理  
	[[picker parentViewController] dismissModalViewControllerAnimated:YES];
	NSLog(@"カメラでの撮影がキャンセルされました");
}


#pragma mark -
#pragma mark CoreLocation delegate

- (void)displayLocation {
	// 現在位置を表示
	[locationLabel setText:[NSString stringWithFormat:@"Lat:%.2f Lon:%.2f Heading:%.0f", appData.coordinate.latitude, appData.coordinate.longitude, appData.heading]]; 
}

- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
	// 緯度経度取得に成功した場合
	appData.coordinate = newLocation.coordinate;
	appData.course = newLocation.course;
	appData.speed = newLocation.speed;
	NSDate *dt = newLocation.timestamp;
	[self displayLocation];
	
	// 取得した座標を中心に地図を表示
	[mapview setCenterCoordinate:appData.coordinate animated:NO];	
}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error{
	// 緯度経度取得に失敗した場合
}

- (void)locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)newHeading {
	appData.heading = newHeading.trueHeading;	// 北が0.0、東が90.0（倍精度浮動小数点）
	[self displayLocation];
}

- (BOOL)locationManagerShouldDisplayHeadingCalibration:(CLLocationManager *)manager {
	// 方位のキャリブレーションを行う
	return YES;
}

@end
